// Traveling Salesman Problem via Hopfield-Tank Neural Network
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the front-end code for the Traveling Salesman Problem.

#region Using declarations 

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Diagnostics;

#endregion

namespace Hopfield_Tank_Neural_Network_Namespace {

	public class TravelingSalesmanProblem : System.Windows.Forms.Form	{

		#region Variables, initialization code, and Main

		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.PictureBox picMap;
		private System.Windows.Forms.ListView listDistances;
		private System.Windows.Forms.TextBox txtName;
		private System.Windows.Forms.Button btnLoadNetwork;
		private System.Windows.Forms.Button btnSaveNetwork;
		private System.Windows.Forms.Button btnClearNetwork;
		private System.Windows.Forms.Button btnAnalyzeStep;
		private System.Windows.Forms.Button btnAnalyzeFull;
		private System.Windows.Forms.Button btnClearSolution;
		private System.Windows.Forms.Button btnScaleCities;
		private System.Windows.Forms.TextBox txtTimeInterval;
		private System.Windows.Forms.TextBox txtdu0;
		private System.Windows.Forms.TextBox txtA;
		private System.Windows.Forms.TextBox txtB;
		private System.Windows.Forms.TextBox txtD;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.ListView listNetworkV;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Windows.Forms.GroupBox grpNetworkV;
		private System.Windows.Forms.Button btnResetSolution;
		private System.Windows.Forms.Button btnHFlip;
		private System.Windows.Forms.Button btnVFlip;
		private System.Windows.Forms.Button btnCWRotate;
		private System.Windows.Forms.Button btnCCWRotate;

		internal HopfieldTankNetwork network;
		private Thread thread = null;
		private System.Timers.Timer timerUpdate;
		private string strCurrentStatus;
		private Random r;
		private string XmlLoadErrors;
		private string strNetworkName = "Sample Network";
		private City movingCity = null;
		private int iMouseX = 0, iMouseY = 0;

		public TravelingSalesmanProblem()	{
			InitializeComponent();
			network = new HopfieldTankNetwork();
			r = new Random();
		}

		#region Windows Form Designer generated code

		protected override void Dispose(bool disposing)	{
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TravelingSalesmanProblem));
			this.picMap = new System.Windows.Forms.PictureBox();
			this.btnClearNetwork = new System.Windows.Forms.Button();
			this.btnLoadNetwork = new System.Windows.Forms.Button();
			this.btnAnalyzeStep = new System.Windows.Forms.Button();
			this.btnAnalyzeFull = new System.Windows.Forms.Button();
			this.btnClearSolution = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.txtName = new System.Windows.Forms.TextBox();
			this.listDistances = new System.Windows.Forms.ListView();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.btnSaveNetwork = new System.Windows.Forms.Button();
			this.btnScaleCities = new System.Windows.Forms.Button();
			this.txtTimeInterval = new System.Windows.Forms.TextBox();
			this.txtdu0 = new System.Windows.Forms.TextBox();
			this.txtA = new System.Windows.Forms.TextBox();
			this.txtB = new System.Windows.Forms.TextBox();
			this.txtD = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.grpNetworkV = new System.Windows.Forms.GroupBox();
			this.listNetworkV = new System.Windows.Forms.ListView();
			this.label2 = new System.Windows.Forms.Label();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.timerUpdate = new System.Timers.Timer();
			this.btnResetSolution = new System.Windows.Forms.Button();
			this.btnHFlip = new System.Windows.Forms.Button();
			this.btnVFlip = new System.Windows.Forms.Button();
			this.btnCWRotate = new System.Windows.Forms.Button();
			this.btnCCWRotate = new System.Windows.Forms.Button();
			this.groupBox1.SuspendLayout();
			this.grpNetworkV.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.timerUpdate)).BeginInit();
			this.SuspendLayout();
			// 
			// picMap
			// 
			this.picMap.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.picMap.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picMap.Location = new System.Drawing.Point(8, 8);
			this.picMap.Name = "picMap";
			this.picMap.Size = new System.Drawing.Size(488, 488);
			this.picMap.TabIndex = 0;
			this.picMap.TabStop = false;
			this.picMap.Paint += new System.Windows.Forms.PaintEventHandler(this.picMap_Paint);
			this.picMap.MouseUp += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseUp);
			this.picMap.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseMove);
			this.picMap.MouseLeave += new System.EventHandler(this.picMap_MouseLeave);
			this.picMap.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picMap_MouseDown);
			// 
			// btnClearNetwork
			// 
			this.btnClearNetwork.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnClearNetwork.Location = new System.Drawing.Point(408, 504);
			this.btnClearNetwork.Name = "btnClearNetwork";
			this.btnClearNetwork.Size = new System.Drawing.Size(88, 23);
			this.btnClearNetwork.TabIndex = 7;
			this.btnClearNetwork.Text = "Clear Map";
			this.btnClearNetwork.Click += new System.EventHandler(this.btnClearNetwork_Click);
			// 
			// btnLoadNetwork
			// 
			this.btnLoadNetwork.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnLoadNetwork.Location = new System.Drawing.Point(664, 208);
			this.btnLoadNetwork.Name = "btnLoadNetwork";
			this.btnLoadNetwork.Size = new System.Drawing.Size(88, 23);
			this.btnLoadNetwork.TabIndex = 13;
			this.btnLoadNetwork.Text = "Load Network";
			this.btnLoadNetwork.Click += new System.EventHandler(this.btnLoadNetwork_Click);
			// 
			// btnAnalyzeStep
			// 
			this.btnAnalyzeStep.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnAnalyzeStep.Location = new System.Drawing.Point(664, 112);
			this.btnAnalyzeStep.Name = "btnAnalyzeStep";
			this.btnAnalyzeStep.Size = new System.Drawing.Size(88, 23);
			this.btnAnalyzeStep.TabIndex = 9;
			this.btnAnalyzeStep.Text = "Analyze (Step)";
			this.btnAnalyzeStep.Click += new System.EventHandler(this.btnAnalyzeStep_Click);
			// 
			// btnAnalyzeFull
			// 
			this.btnAnalyzeFull.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnAnalyzeFull.Location = new System.Drawing.Point(664, 144);
			this.btnAnalyzeFull.Name = "btnAnalyzeFull";
			this.btnAnalyzeFull.Size = new System.Drawing.Size(88, 23);
			this.btnAnalyzeFull.TabIndex = 10;
			this.btnAnalyzeFull.Text = "Analyze (Full)";
			this.btnAnalyzeFull.Click += new System.EventHandler(this.btnAnalyzeFull_Click);
			// 
			// btnClearSolution
			// 
			this.btnClearSolution.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnClearSolution.Location = new System.Drawing.Point(664, 48);
			this.btnClearSolution.Name = "btnClearSolution";
			this.btnClearSolution.Size = new System.Drawing.Size(88, 23);
			this.btnClearSolution.TabIndex = 8;
			this.btnClearSolution.Text = "Clear Network";
			this.btnClearSolution.Click += new System.EventHandler(this.btnClearSolution_Click);
			// 
			// label1
			// 
			this.label1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label1.Location = new System.Drawing.Point(512, 48);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 23);
			this.label1.TabIndex = 7;
			this.label1.Text = "Name";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtName
			// 
			this.txtName.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtName.Location = new System.Drawing.Point(552, 48);
			this.txtName.Name = "txtName";
			this.txtName.Size = new System.Drawing.Size(104, 20);
			this.txtName.TabIndex = 0;
			this.txtName.Text = "Sample Network";
			this.txtName.TextChanged += new System.EventHandler(this.txtName_TextChanged);
			// 
			// listDistances
			// 
			this.listDistances.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.listDistances.AutoArrange = false;
			this.listDistances.Font = new System.Drawing.Font("Microsoft Sans Serif", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.listDistances.FullRowSelect = true;
			this.listDistances.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listDistances.Location = new System.Drawing.Point(8, 16);
			this.listDistances.MultiSelect = false;
			this.listDistances.Name = "listDistances";
			this.listDistances.Size = new System.Drawing.Size(264, 168);
			this.listDistances.TabIndex = 12;
			this.listDistances.View = System.Windows.Forms.View.Details;
			// 
			// groupBox1
			// 
			this.groupBox1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.groupBox1.Controls.Add(this.listDistances);
			this.groupBox1.Location = new System.Drawing.Point(760, 40);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(280, 192);
			this.groupBox1.TabIndex = 14;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Distances";
			this.groupBox1.Resize += new System.EventHandler(this.groupBox1_Resize);
			// 
			// btnSaveNetwork
			// 
			this.btnSaveNetwork.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnSaveNetwork.Location = new System.Drawing.Point(664, 176);
			this.btnSaveNetwork.Name = "btnSaveNetwork";
			this.btnSaveNetwork.Size = new System.Drawing.Size(88, 23);
			this.btnSaveNetwork.TabIndex = 12;
			this.btnSaveNetwork.Text = "Save Network";
			this.btnSaveNetwork.Click += new System.EventHandler(this.btnSaveNetwork_Click);
			// 
			// btnScaleCities
			// 
			this.btnScaleCities.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnScaleCities.Location = new System.Drawing.Point(328, 504);
			this.btnScaleCities.Name = "btnScaleCities";
			this.btnScaleCities.Size = new System.Drawing.Size(72, 23);
			this.btnScaleCities.TabIndex = 14;
			this.btnScaleCities.Text = "Scale";
			this.btnScaleCities.Click += new System.EventHandler(this.btnScaleCities_Click);
			// 
			// txtTimeInterval
			// 
			this.txtTimeInterval.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtTimeInterval.Location = new System.Drawing.Point(552, 112);
			this.txtTimeInterval.Name = "txtTimeInterval";
			this.txtTimeInterval.Size = new System.Drawing.Size(104, 20);
			this.txtTimeInterval.TabIndex = 2;
			this.txtTimeInterval.Text = "0.0001";
			this.txtTimeInterval.TextChanged += new System.EventHandler(this.txtTimeInterval_TextChanged);
			// 
			// txtdu0
			// 
			this.txtdu0.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtdu0.Location = new System.Drawing.Point(552, 80);
			this.txtdu0.Name = "txtdu0";
			this.txtdu0.Size = new System.Drawing.Size(104, 20);
			this.txtdu0.TabIndex = 1;
			this.txtdu0.Text = "0.001";
			this.txtdu0.TextChanged += new System.EventHandler(this.txtdu0_TextChanged);
			// 
			// txtA
			// 
			this.txtA.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtA.Location = new System.Drawing.Point(552, 144);
			this.txtA.Name = "txtA";
			this.txtA.Size = new System.Drawing.Size(104, 20);
			this.txtA.TabIndex = 3;
			this.txtA.Text = "1.2";
			this.txtA.TextChanged += new System.EventHandler(this.txtA_TextChanged);
			// 
			// txtB
			// 
			this.txtB.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtB.Location = new System.Drawing.Point(552, 176);
			this.txtB.Name = "txtB";
			this.txtB.Size = new System.Drawing.Size(104, 20);
			this.txtB.TabIndex = 4;
			this.txtB.Text = "1.2";
			this.txtB.TextChanged += new System.EventHandler(this.txtB_TextChanged);
			// 
			// txtD
			// 
			this.txtD.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtD.Location = new System.Drawing.Point(552, 208);
			this.txtD.Name = "txtD";
			this.txtD.Size = new System.Drawing.Size(104, 20);
			this.txtD.TabIndex = 6;
			this.txtD.Text = "1.5";
			this.txtD.TextChanged += new System.EventHandler(this.txtD_TextChanged);
			// 
			// label3
			// 
			this.label3.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label3.Location = new System.Drawing.Point(512, 80);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(32, 23);
			this.label3.TabIndex = 25;
			this.label3.Text = "du0";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label5
			// 
			this.label5.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label5.Location = new System.Drawing.Point(512, 144);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(32, 23);
			this.label5.TabIndex = 27;
			this.label5.Text = "A";
			this.label5.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label6
			// 
			this.label6.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label6.Location = new System.Drawing.Point(512, 176);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(32, 23);
			this.label6.TabIndex = 28;
			this.label6.Text = "B";
			this.label6.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label8
			// 
			this.label8.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label8.Location = new System.Drawing.Point(512, 208);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(32, 23);
			this.label8.TabIndex = 30;
			this.label8.Text = "D";
			this.label8.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label4
			// 
			this.label4.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label4.Location = new System.Drawing.Point(512, 112);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(32, 23);
			this.label4.TabIndex = 26;
			this.label4.Text = "dt";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// grpNetworkV
			// 
			this.grpNetworkV.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.grpNetworkV.Controls.Add(this.listNetworkV);
			this.grpNetworkV.Location = new System.Drawing.Point(504, 240);
			this.grpNetworkV.Name = "grpNetworkV";
			this.grpNetworkV.Size = new System.Drawing.Size(536, 288);
			this.grpNetworkV.TabIndex = 16;
			this.grpNetworkV.TabStop = false;
			this.grpNetworkV.Text = "Network (V and u)";
			// 
			// listNetworkV
			// 
			this.listNetworkV.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.listNetworkV.AutoArrange = false;
			this.listNetworkV.Font = new System.Drawing.Font("Microsoft Sans Serif", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.listNetworkV.FullRowSelect = true;
			this.listNetworkV.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listNetworkV.Location = new System.Drawing.Point(8, 16);
			this.listNetworkV.MultiSelect = false;
			this.listNetworkV.Name = "listNetworkV";
			this.listNetworkV.Size = new System.Drawing.Size(520, 264);
			this.listNetworkV.TabIndex = 12;
			this.listNetworkV.View = System.Windows.Forms.View.Details;
			this.listNetworkV.Resize += new System.EventHandler(this.listNetworkV_Resize);
			// 
			// label2
			// 
			this.label2.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label2.Location = new System.Drawing.Point(504, 8);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(56, 23);
			this.label2.TabIndex = 31;
			this.label2.Text = "Status:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtStatus
			// 
			this.txtStatus.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.txtStatus.Location = new System.Drawing.Point(560, 8);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(480, 20);
			this.txtStatus.TabIndex = 32;
			this.txtStatus.Text = "";
			// 
			// timerUpdate
			// 
			this.timerUpdate.Enabled = true;
			this.timerUpdate.Interval = 200;
			this.timerUpdate.SynchronizingObject = this;
			this.timerUpdate.Elapsed += new System.Timers.ElapsedEventHandler(this.timerUpdate_Elapsed);
			// 
			// btnResetSolution
			// 
			this.btnResetSolution.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.btnResetSolution.Location = new System.Drawing.Point(664, 80);
			this.btnResetSolution.Name = "btnResetSolution";
			this.btnResetSolution.Size = new System.Drawing.Size(88, 23);
			this.btnResetSolution.TabIndex = 33;
			this.btnResetSolution.Text = "Reset Network";
			this.btnResetSolution.Click += new System.EventHandler(this.btnResetSolution_Click);
			// 
			// btnHFlip
			// 
			this.btnHFlip.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnHFlip.Location = new System.Drawing.Point(8, 504);
			this.btnHFlip.Name = "btnHFlip";
			this.btnHFlip.Size = new System.Drawing.Size(64, 24);
			this.btnHFlip.TabIndex = 34;
			this.btnHFlip.Text = "H Flip";
			this.btnHFlip.Click += new System.EventHandler(this.btnHFlip_Click);
			// 
			// btnVFlip
			// 
			this.btnVFlip.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnVFlip.Location = new System.Drawing.Point(80, 504);
			this.btnVFlip.Name = "btnVFlip";
			this.btnVFlip.Size = new System.Drawing.Size(64, 24);
			this.btnVFlip.TabIndex = 35;
			this.btnVFlip.Text = "V Flip";
			this.btnVFlip.Click += new System.EventHandler(this.btnVFlip_Click);
			// 
			// btnCWRotate
			// 
			this.btnCWRotate.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnCWRotate.Location = new System.Drawing.Point(152, 504);
			this.btnCWRotate.Name = "btnCWRotate";
			this.btnCWRotate.Size = new System.Drawing.Size(80, 24);
			this.btnCWRotate.TabIndex = 36;
			this.btnCWRotate.Text = "CW Rotate";
			this.btnCWRotate.Click += new System.EventHandler(this.btnCWRotate_Click);
			// 
			// btnCCWRotate
			// 
			this.btnCCWRotate.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.btnCCWRotate.Location = new System.Drawing.Point(240, 504);
			this.btnCCWRotate.Name = "btnCCWRotate";
			this.btnCCWRotate.Size = new System.Drawing.Size(80, 24);
			this.btnCCWRotate.TabIndex = 37;
			this.btnCCWRotate.Text = "CCW Rotate";
			this.btnCCWRotate.Click += new System.EventHandler(this.btnCCWRotate_Click);
			// 
			// TravelingSalesmanProblem
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(1048, 534);
			this.Controls.Add(this.btnCCWRotate);
			this.Controls.Add(this.btnCWRotate);
			this.Controls.Add(this.btnVFlip);
			this.Controls.Add(this.btnHFlip);
			this.Controls.Add(this.btnResetSolution);
			this.Controls.Add(this.txtStatus);
			this.Controls.Add(this.txtD);
			this.Controls.Add(this.txtB);
			this.Controls.Add(this.txtA);
			this.Controls.Add(this.txtdu0);
			this.Controls.Add(this.txtTimeInterval);
			this.Controls.Add(this.txtName);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label8);
			this.Controls.Add(this.label6);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.btnScaleCities);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnClearSolution);
			this.Controls.Add(this.btnAnalyzeFull);
			this.Controls.Add(this.btnAnalyzeStep);
			this.Controls.Add(this.btnSaveNetwork);
			this.Controls.Add(this.btnLoadNetwork);
			this.Controls.Add(this.btnClearNetwork);
			this.Controls.Add(this.picMap);
			this.Controls.Add(this.grpNetworkV);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "TravelingSalesmanProblem";
			this.Text = "Traveling Salesman Problem via Hopfield-Tank Neural Network";
			this.Resize += new System.EventHandler(this.TravelingSalesmanProblem_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.TravelingSalesmanProblem_Closing);
			this.groupBox1.ResumeLayout(false);
			this.grpNetworkV.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.timerUpdate)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new TravelingSalesmanProblem());
		}

		#endregion

		#region Button-click event handlers

		private void txtName_TextChanged(object sender, System.EventArgs e) {
			network.name = txtName.Text;
		}

		private void btnClearNetwork_Click(object sender, System.EventArgs e) {
			network.cities = new ArrayList();
			strNetworkName = txtName.Text = "Sample Network";
			picMap.Refresh();
			SetDistanceList();
			SetNetworkLists();
		}

		private void btnScaleCities_Click(object sender, System.EventArgs e) {
			// find largest coordinates
			double dMaxX = 0;
			double dMaxY = 0;
			double dMinX = picMap.Width;
			double dMinY = picMap.Height;
			foreach (object cityobject in network.cities) {
				dMaxX = Math.Max(dMaxX, ((City) cityobject).x);
				dMaxY = Math.Max(dMaxY, ((City) cityobject).y);
				dMinX = Math.Min(dMinX, ((City) cityobject).x);
				dMinY = Math.Min(dMinY, ((City) cityobject).y);
			}
			if (dMaxX == dMinX) {
				dMinX = picMap.Width / 2;
				dMaxX = 0;
			}
			else
				dMaxX = (picMap.Width - 40) / (dMaxX - dMinX);
			if (dMaxY == dMinY) {
				dMinY = picMap.Height / 2;
				dMaxY = 0;
			}
			else
				dMaxY = (picMap.Height - 40) / (dMaxY - dMinY);
			// adjust cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				city.x = 20 + (city.x - dMinX) * dMaxX;
				city.y = 20 + (city.y - dMinY) * dMaxY;
			}
			// show new coords, etc.
			picMap.Refresh();
			listDistances.Refresh();
		}

		private void txtTimeInterval_TextChanged(object sender, System.EventArgs e) {
			try {
				network.dTimeInterval = Double.Parse(txtTimeInterval.Text);
			} catch (Exception ex) {}
		}

		private void btnAnalyzeStep_Click(object sender, System.EventArgs e) {
			txtStatus.Text = network.Analyze();
			txtStatus.Refresh();
			SetNetworkLists();
			picMap.Refresh();
		}

		private void btnClearSolution_Click(object sender, System.EventArgs e) {
			network.Initialize(r.Next(99999));
			SetNetworkLists();
			picMap.Refresh();
		}

		private void txtdu0_TextChanged(object sender, System.EventArgs e) {
			try { network.u0variation = double.Parse(txtdu0.Text); } catch (Exception ex) {}
		}

		private void txtA_TextChanged(object sender, System.EventArgs e) {
			try { network.A = double.Parse(txtA.Text); } catch (Exception ex) {}
		}

		private void txtB_TextChanged(object sender, System.EventArgs e) {
			try { network.B = double.Parse(txtB.Text); } catch (Exception ex) {}
		}

		private void txtD_TextChanged(object sender, System.EventArgs e) {
			try { network.D = double.Parse(txtD.Text); } catch (Exception ex) {}
		}

		private void btnAnalyzeFull_Click(object sender, System.EventArgs e) {
			if (thread == null) {  // fire up thread
				thread = new Thread(new ThreadStart(AnalyzeFullThread));
				thread.Start();
				btnAnalyzeFull.Text = "Abort";
				btnAnalyzeFull.Refresh();
			}
			else {  // abort thread
				thread.Abort();
				thread = null;
				txtStatus.Text = "Aborted. Total distance = " + network.TotalDistance();
				txtStatus.Refresh();
				btnAnalyzeFull.Text = "Analyze (Full)";
				btnAnalyzeFull.Refresh();
				SetNetworkLists();
			}
		}

		private void btnResetSolution_Click(object sender, System.EventArgs e) {
			network.Reset();
			SetNetworkLists();
		}

		private void btnHFlip_Click(object sender, System.EventArgs e) {
			foreach (City c in network.cities)
				c.x = picMap.Width - c.x;
			picMap.Refresh();
			SetNetworkLists();
		}

		private void btnVFlip_Click(object sender, System.EventArgs e) {
			foreach (City c in network.cities)
				c.y = picMap.Height - c.y;
			picMap.Refresh();
			SetNetworkLists();
		}

		private void btnCWRotate_Click(object sender, System.EventArgs e) {
			foreach (City c in network.cities) {
				double newX = picMap.Width * ((float) (picMap.Height - c.y) / (float) picMap.Height);
				double newY = picMap.Height * ((float) c.x / (float) picMap.Width);
				c.y = newY;
				c.x = newX;
			}
			picMap.Refresh();
			SetDistanceList();
			SetNetworkLists();
		}

		private void btnCCWRotate_Click(object sender, System.EventArgs e) {
			foreach (City c in network.cities) {
				double newX = picMap.Width * ((float) c.y / (float) picMap.Height);
				double newY = picMap.Height * ((float) (picMap.Width - c.x) / (float) picMap.Width);
				c.y = newY;
				c.x = newX;
			}
			picMap.Refresh();
			SetDistanceList();
			SetNetworkLists();
		}

		#endregion

		#region XML read/write functions and event handlers

		private void btnSaveNetwork_Click(object sender, System.EventArgs e) {
			SaveFileDialog s = new SaveFileDialog();
			s.Filter = "Network Files (*.net)|*.net";
			s.OverwritePrompt = true;
			if (s.ShowDialog() == DialogResult.OK) {
				XmlTextWriter save = new XmlTextWriter(s.FileName, System.Text.Encoding.UTF8);
				save.Formatting = Formatting.Indented;
				save.IndentChar = ' ';
				save.WriteStartDocument();
				save.WriteStartElement("HopfieldTankNetwork");
				save.Indentation++;
				// write parameters
				save.WriteStartElement("Parameters");
				save.Indentation++;
				save.WriteElementString("NetworkName", network.name);
				save.WriteElementString("A", network.A.ToString());
				save.WriteElementString("B", network.B.ToString());
				save.WriteElementString("C", network.C.ToString());
				save.WriteElementString("D", network.D.ToString());
				save.WriteElementString("du0", network.u0variation.ToString());
				save.WriteElementString("dt", network.dTimeInterval.ToString());
				save.Indentation--;
				save.WriteEndElement();
				// write neuron data
				save.WriteStartElement("Cities");
				save.Indentation++;
				foreach (object cityobject in network.cities) {
					City city = (City) cityobject;
					save.WriteStartElement("City");
					save.Indentation++;
					save.WriteElementString("Name", city.name);
					save.WriteElementString("X", city.x.ToString());
					save.WriteElementString("Y", city.y.ToString());
					save.Indentation--;
					save.WriteEndElement();
				}
				save.Indentation--;
				save.WriteEndElement();
				// close XML file
				save.Indentation--;
				save.WriteEndElement();
				save.WriteEndDocument();
				save.Close();
			}
		}

		private void btnLoadNetwork_Click(object sender, System.EventArgs e) {
			City city;
			network = new HopfieldTankNetwork();
			network.cities = new ArrayList();
			OpenFileDialog open = new OpenFileDialog();
			open.Filter = "Network files (*.net)|*.net";
			if (open.ShowDialog() == DialogResult.OK) {  // selected an XML file
				try {
					XmlTextReader xtr = new XmlTextReader(open.FileName);
					XmlValidatingReader xvr = new XmlValidatingReader(xtr);
					xvr.ValidationType = ValidationType.Schema;
					xvr.ValidationEventHandler += new ValidationEventHandler(ValidationErrors);
					XmlSchemaCollection xsc = new XmlSchemaCollection();
					xsc.Add("", System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 61) + @"Schema Files\Network.xsd");
					xvr.Schemas.Add(xsc);
					XmlDocument xd = new XmlDocument();
					XmlLoadErrors = "";
					xd.Load(xvr);
					if (XmlLoadErrors.Length > 0)
						MessageBox.Show("Errors loading network document. This does not appear to be a valid network file.", "Errors Loading Network Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
					else {
						foreach (XmlNode x1 in xd.ChildNodes) {
							if (x1.Name == "HopfieldTankNetwork") {
								foreach (XmlNode x2 in x1.ChildNodes) {
									if (x2.Name == "Parameters") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "NetworkName")
												network.name = x3.InnerText;
											else if (x3.Name == "A")
												network.A = double.Parse(x3.InnerText);
											else if (x3.Name == "B")
												network.B = double.Parse(x3.InnerText);
											else if (x3.Name == "C")
												network.C = double.Parse(x3.InnerText);
											else if (x3.Name == "D")
												network.D = double.Parse(x3.InnerText);
											else if (x3.Name == "du0")
												network.u0variation = double.Parse(x3.InnerText);
											else if (x3.Name == "dt")
												network.dTimeInterval = double.Parse(x3.InnerText);
										}
									}
									else if (x2.Name == "Cities") {
										foreach (XmlNode x3 in x2.ChildNodes) {
											if (x3.Name == "City") {
												city = new City(100, 100);
												foreach (XmlNode x4 in x3.ChildNodes) {
													if (x4.Name == "Name")
														city.name = x4.InnerText;
													else if (x4.Name == "X")
														city.x = Double.Parse(x4.InnerText);
													else if (x4.Name == "Y")
														city.y = Double.Parse(x4.InnerText);
												}
												network.cities.Add(city);
											}
										}
									}
								}
							}
						}
						xtr.Close();
						txtName.Text = network.name;
						txtA.Text = network.A.ToString();
						txtB.Text = network.B.ToString();
						txtD.Text = network.D.ToString();
						txtdu0.Text = network.u0variation.ToString();
						txtTimeInterval.Text = network.dTimeInterval.ToString();
						network.Initialize((new Random()).Next(99999));
						SetDistanceList();
						SetNetworkLists();
						picMap.Refresh();
					}
				} catch (Exception ex) {
					MessageBox.Show(ex.Message);
				}
			}
		}
		public void ValidationErrors(object sender, ValidationEventArgs e) {
			XmlLoadErrors += e.Message + "\r\n";
		}

		#endregion

		#region Other event handlers

		private void picMap_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			iMouseX = e.X;
			iMouseY = e.Y;
			if (movingCity != null) {
				movingCity.x = iMouseX;
				movingCity.y = iMouseY;
				picMap.Refresh();
				network.SetDistances();
				SetDistanceList();
				SetNetworkLists();
			}
		}

		private void picMap_MouseLeave(object sender, System.EventArgs e) {
			movingCity = null;
		}

		private void picMap_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			City clickedCity = null;
			// check location first
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				if (Math.Abs(city.x - iMouseX) + Math.Abs(city.y - iMouseY) < 15) {  // close enough - delete city.
					clickedCity = city;
					break;
				}
			}
			if (e.Button == MouseButtons.Left) {  // deleting or placing city
				if (clickedCity != null)
					network.cities.Remove(clickedCity);
				else {  // placing new city
					int x = iMouseX;
					int y = iMouseY;  // preserve coordinates
					City city = new City(x, y);
					int i = (new Random()).Next(139);
					switch (i) {
						case 1: city.name = "Baltimore"; break;
						case 2: city.name = "Chicago"; break;
						case 3: city.name = "Cincinnati"; break;
						case 4: city.name = "New York City"; break;
						case 5: city.name = "Kansas City"; break;
						case 6: city.name = "San Diego"; break;
						case 7: city.name = "San Francisco"; break;
						case 8: city.name = "Paris"; break;
						case 9: city.name = "Berlin"; break;
						case 10: city.name = "Washington D.C."; break;
						case 11: city.name = "Cincinnati"; break;
						case 12: city.name = "Columbus"; break;
						case 13: city.name = "Philadelphia"; break;
						case 14: city.name = "New Orleans"; break;
						case 15: city.name = "Dallas"; break;
						case 16: city.name = "Salt Lake City"; break;
						case 17: city.name = "Houston"; break;
						case 18: city.name = "Los Angeles"; break;
						case 19: city.name = "Boston"; break;
						case 20: city.name = "Detroit"; break;
						case 21: city.name = "Toronto"; break;
						case 22: city.name = "Albany"; break;
						case 23: city.name = "Orlando"; break;
						case 24: city.name = "Anchorage"; break;
						case 25: city.name = "Brussels"; break;
						case 26: city.name = "London"; break;
						case 27: city.name = "Denver"; break;
						case 28: city.name = "San Antonio"; break;
						case 29: city.name = "Tampa"; break;
						case 30: city.name = "Miami"; break;
						case 31: city.name = "Atlanta"; break;
						case 32: city.name = "Providence"; break;
						case 33: city.name = "Tallahassee"; break;
						case 34: city.name = "Nashville"; break;
						case 35: city.name = "Las Vegas"; break;
						case 36: city.name = "Santa Fe"; break;
						case 37: city.name = "Boise"; break;
						case 38: city.name = "Seattle"; break;
						case 39: city.name = "Portland"; break;
						case 40: city.name = "Bismarck"; break;
						case 41: city.name = "Minneapolis"; break;
						case 42: city.name = "Milwaukee"; break;
						case 43: city.name = "Montreal"; break;
						case 44: city.name = "Winnipeg"; break;
						case 45: city.name = "Calgary"; break;
						case 46: city.name = "Birmingham"; break;
						case 47: city.name = "St. Louis"; break;
						case 48: city.name = "Little Rock"; break;
						case 49: city.name = "Oklahoma City"; break;
						case 50: city.name = "Indianapolis"; break;
						case 51: city.name = "Pittsburgh"; break;
						case 52: city.name = "Phoenix"; break;
						case 53: city.name = "Omaha"; break;
						case 54: city.name = "Wichita"; break;
						case 55: city.name = "Lexington"; break;
						case 56: city.name = "Louisville"; break;
						case 57: city.name = "Richmond"; break;
						case 58: city.name = "Charlotte"; break;
						case 59: city.name = "Savannah"; break;
						case 60: city.name = "Stalingrad"; break;
						case 61: city.name = "Leningrad"; break;
						case 62: city.name = "Moscow"; break;
						case 63: city.name = "Baghdad"; break;
						case 64: city.name = "Grand Rapids"; break;
						case 65: city.name = "Ottawa"; break;
						case 66: city.name = "Guatemala City"; break;
						case 67: city.name = "Mexico City"; break;
						case 68: city.name = "Havana"; break;
						case 69: city.name = "San Juan"; break;
						case 70: city.name = "Rio de Janeiro"; break;
						case 71: city.name = "Madrid"; break;
						case 72: city.name = "Barcelona"; break;
						case 73: city.name = "Marseilles"; break;
						case 74: city.name = "Amsterdam"; break;
						case 75: city.name = "Dublin"; break;
						case 76: city.name = "Glasgow"; break;
						case 77: city.name = "Frankfurt"; break;
						case 78: city.name = "Hamburg"; break;
						case 79: city.name = "Munich"; break;
						case 80: city.name = "Milan"; break;
						case 81: city.name = "Rome"; break;
						case 82: city.name = "Naples"; break;
						case 83: city.name = "Genoa"; break;
						case 84: city.name = "Belgrade"; break;
						case 85: city.name = "Zagreb"; break;
						case 86: city.name = "Sofia"; break;
						case 87: city.name = "Athens"; break;
						case 88: city.name = "Istanbul"; break;
						case 89: city.name = "Budapest"; break;
						case 90: city.name = "Warsaw"; break;
						case 91: city.name = "Kiev"; break;
						case 92: city.name = "Belarus"; break;
						case 93: city.name = "Stockholm"; break;
						case 94: city.name = "Oslo"; break;
						case 95: city.name = "Minsk"; break;
						case 96: city.name = "Kolkata"; break;
						case 97: city.name = "Beijing"; break;
						case 98: city.name = "Hong Kong"; break;
						case 99: city.name = "Tokyo"; break;
						case 100: city.name = "Mumbai"; break;
						case 101: city.name = "Bhopal"; break;
						case 102: city.name = "Delhi"; break;
						case 103: city.name = "Kabul"; break;
						case 104: city.name = "Mosul"; break;
						case 105: city.name = "Jerusalem"; break;
						case 106: city.name = "Damascus"; break;
						case 107: city.name = "Riyadh"; break;
						case 108: city.name = "Mecca"; break;
						case 109: city.name = "Dubai"; break;
						case 110: city.name = "Alexandria"; break;
						case 111: city.name = "Cairo"; break;
						case 112: city.name = "Tunis"; break;
						case 113: city.name = "Tripoli"; break;
						case 114: city.name = "Algiers"; break;
						case 115: city.name = "Beirut"; break;
						case 116: city.name = "Bangkok"; break;
						case 117: city.name = "Ho Chi Minh City"; break;
						case 118: city.name = "Hanoi"; break;
						case 119: city.name = "Chengdu"; break;
						case 120: city.name = "Guangzhou"; break;
						case 121: city.name = "Seoul"; break;
						case 122: city.name = "Shanghai"; break;
						case 123: city.name = "Phnom Penh"; break;
						case 124: city.name = "Mandalay"; break;
						case 125: city.name = "Kuala Lumpur"; break;
						case 126: city.name = "Singapore"; break;
						case 127: city.name = "Jakarta"; break;
						case 128: city.name = "Perth"; break;
						case 129: city.name = "Melbourne"; break;
						case 130: city.name = "Sydney"; break;
						case 131: city.name = "Brisbane"; break;
						case 132: city.name = "Hokkaido"; break;
						case 133: city.name = "Kyoto"; break;
						case 134: city.name = "Kagashima"; break;
						case 135: city.name = "Fukuoka"; break;
						case 136: city.name = "Murmansk"; break;
						case 137: city.name = "Turin"; break;
						case 138: city.name = "Lisbon"; break;
						default: city.name = "Cleveland"; break;
					}
					network.cities.Add(city);
				}
				network.Initialize(r.Next());
				SetDistanceList();
				SetNetworkLists();
			}
			else if ((e.Button == MouseButtons.Right) && (clickedCity != null))  // start moving city
				movingCity = clickedCity;
			picMap.Refresh();
		}

		private void picMap_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
			if ((e.Button == MouseButtons.Right) && (movingCity != null)) {
				movingCity = null;
				network.Initialize(r.Next());
				SetDistanceList();
				SetNetworkLists();
				picMap.Refresh();
			}
		}

		private void picMap_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.BlanchedAlmond, e.ClipRectangle);

			// draw routes
			if (network.cities.Count > 1) {

				int iNextCity = 0;
				for (int X = 1; X < network.cities.Count; X++) {
					if (network.V[X, 0] > network.V[iNextCity, 0])
						iNextCity = X;
				}
				int iCurrentCity;
				for (int i = network.cities.Count - 1; i >= 0; i--) {
					iCurrentCity = 0;
					for (int X = 1; X < network.cities.Count; X++) {
						if (network.V[X, i] > network.V[iCurrentCity, i])
							iCurrentCity = X;
					}
					if ((network.V[iNextCity, (i + 1) % network.cities.Count] > 0.05) && (network.V[iCurrentCity, i] > 0.05)) {
						Pen pen = new Pen(Color.FromArgb((int) (network.V[iNextCity, (i + 1) % network.cities.Count] * 128) + 127, 0, 255, 0));
						if (network.V[iNextCity, (i + 1) % network.cities.Count] > 0.95)
							pen = new Pen(Color.FromArgb(255, 0, 0));
						g.DrawLine(pen, (int) ((City) network.cities[iCurrentCity]).x, (int) ((City) network.cities[iCurrentCity]).y, (int) ((City) network.cities[iNextCity]).x, (int) ((City) network.cities[iNextCity]).y);
					}
					iNextCity = iCurrentCity;
				}
			}

			// draw cities
			foreach (object cityobject in network.cities) {
				City city = (City) cityobject;
				g.FillEllipse(Brushes.Tan, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawEllipse(Pens.Black, (int) (city.x - 5), (int) (city.y - 5), 10, 10);
				g.DrawString(city.name, font, Brushes.Black, (int) (city.x), (int) (city.y - 20), format);
			}
		}

		private void timerUpdate_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			if (thread != null) {
				picMap.Refresh();
				txtStatus.Text = strCurrentStatus;
				txtStatus.Refresh();
			}
		}

		private void TravelingSalesmanProblem_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (thread != null)
				thread.Abort();
		}

		private void listNetworkV_Resize(object sender, System.EventArgs e) {
			SetNetworkLists();
		}

		private void groupBox1_Resize(object sender, System.EventArgs e) {
			SetNetworkLists();
		}

		private void TravelingSalesmanProblem_Resize(object sender, System.EventArgs e) {
			SetDistanceList();
			SetNetworkLists();
		}

		#endregion

		#region Helper functions

		private void SetDistanceList() {
			listDistances.BeginUpdate();
			if (network.cities.Count == 0) {  // zero the list
				listDistances.Columns.Clear();
				listDistances.Items.Clear();
			}
			else {
				// setup list columns
				int width = listDistances.Width / (network.cities.Count + 1);
				listDistances.Columns.Clear();
				ColumnHeader ch = new ColumnHeader();
				ch.Width = width;
				ch.Text = "Name";
				listDistances.Columns.Add(ch);
				foreach (object cityobject in network.cities) {
					ColumnHeader c = new ColumnHeader();
					c.Width = width;
					c.Text = ((City) cityobject).name;
					listDistances.Columns.Add(c);
				}
				// setup list rows
				listDistances.Items.Clear();
				for (int i = 0; i < network.cities.Count; i++) {
					City city = (City) network.cities[i];
					ListViewItem row = new ListViewItem(city.name);
					for (int j = 0; j < network.cities.Count; j++)
						row.SubItems.Add("" + network.distances[i, j]);
					listDistances.Items.Add(row);
				}
			}
			listDistances.EndUpdate();
			listDistances.Refresh();
		}

		private void SetNetworkLists() {
			// update network V
			listNetworkV.BeginUpdate();
			if (network.cities.Count == 0) {  // zero the list
				listNetworkV.Columns.Clear();
				listNetworkV.Items.Clear();
			}
			else {
				// setup list columns
				int width = listNetworkV.Width / (network.cities.Count + 1);
				listNetworkV.Columns.Clear();
				ColumnHeader ch = new ColumnHeader();
				ch.Width = width;
				ch.Text = "Name";
				listNetworkV.Columns.Add(ch);
				for (int i = 0; i < network.cities.Count; i++) {
					ColumnHeader c = new ColumnHeader();
					c.Text = "" + (i + 1);
					c.Width = width;
					listNetworkV.Columns.Add(c);
				}
				// setup list rows
				listNetworkV.Items.Clear();
				for (int i = 0; i < network.cities.Count; i++) {
					City city = (City) network.cities[i];
					ListViewItem row = new ListViewItem(city.name);
					for (int j = 0; j < network.cities.Count; j++)
						row.SubItems.Add("" + Math.Round(network.V[i,j], 3));
					listNetworkV.Items.Add(row);
				}
				listNetworkV.Items.Add(new ListViewItem(new string[] {} ));
				for (int i = 0; i < network.cities.Count; i++) {
					City city = (City) network.cities[i];
					ListViewItem row = new ListViewItem(city.name);
					for (int j = 0; j < network.cities.Count; j++)
						row.SubItems.Add("" + Math.Round(network.u[i,j], 3));
					listNetworkV.Items.Add(row);
				}
			}
			listNetworkV.EndUpdate();
			listNetworkV.Refresh();
		}

		private void AnalyzeFull() {
			int totalsteps = 0;
			do {
				totalsteps++;
				network.Analyze();
			} while ((network.E != 0.0) && (totalsteps < 10000));
		}

		private void AnalyzeFullThread() {
			int totalsteps = 0;
			do {
				totalsteps++;
				network.Analyze();
				strCurrentStatus = "Status: Analyzing. Steps = " + totalsteps + ". Total distance = " + network.TotalDistance();
			} while ((network.E != 0.0) && (totalsteps < 50000));
			if (network.E == 0.0)
				txtStatus.Text = strCurrentStatus = "Status: Finished. Total distance = " + network.TotalDistance();
			else
				txtStatus.Text = strCurrentStatus = "Status: Terminated after " + totalsteps + " steps. Total distance = " + network.TotalDistance() + ".";
			txtStatus.Refresh();
			btnAnalyzeFull.Text = "Analyze (Full)";
			btnAnalyzeFull.Refresh();
			picMap.Refresh();
			SetNetworkLists();
			thread = null;
		}

		#endregion

	}
}
